from importlib.metadata import version, PackageNotFoundError
from .maiconverter import main

try:
    __version__ = version("maiconverter")
except PackageNotFoundError:
    __version__ = "not installed"



import os
import gzip
from typing import Union
from binascii import unhexlify
from Crypto.Cipher import AES


def finale_db_decrypt(
    key: str,
    path: str,
    encoding: str = "utf-16-le",
    ignore_errors: bool = True,
) -> str:
    if os.path.getsize(path) % 0x10 != 0:
        raise ValueError("Ciphertext is not a multiple of 16")

    with open(path, "rb") as encrypted_file:
        iv = encrypted_file.read(0x10)
        ciphertext = encrypted_file.read()

    return finale_decrypt(
        mode="db",
        key=key,
        iv=iv,
        ciphertext=ciphertext,
        encoding=encoding,
        ignore_errors=ignore_errors,
    )


def finale_db_encrypt(
    key: str,
    path: str,
    encoding: str = "utf-16-le",
) -> bytes:
    with open(path, "r", encoding="utf-8") as f:
        rawtext = f.read()

    return finale_encrypt(mode="db", key=key, plaintext=rawtext, encoding=encoding)


def finale_chart_decrypt(key: str, path: str, encoding: str = "utf-8") -> str:
    if os.path.getsize(path) % 0x10 != 0:
        raise ValueError("Ciphertext is not a multiple of 16")

    with open(path, "rb") as encrypted_file:
        iv = encrypted_file.read(0x10)
        ciphertext = encrypted_file.read()

    key_bin = int(key, 0).to_bytes(0x10, "big")

    return finale_decrypt(
        mode="chart",
        key=key_bin,
        iv=iv,
        ciphertext=ciphertext,
        encoding=encoding,
    )


def finale_chart_encrypt(key: str, path: str, encoding: str = "utf-8") -> bytes:
    key_bin = int(key, 0).to_bytes(0x10, "big")

    with open(path, "r", encoding="utf-8") as f:
        rawtext = f.read()

    return finale_encrypt(
        mode="chart", key=key_bin, plaintext=rawtext, encoding=encoding
    )


def finale_decrypt(
    mode: str,
    key: Union[str, bytes],
    iv: bytes,
    ciphertext: bytes,
    encoding: str,
    ignore_errors: bool = True,
) -> str:
    if not isinstance(key, bytes):
        key = int(key, 0).to_bytes(0x10, "big")

    cipher = AES.new(key, AES.MODE_CBC, iv)
    gzipdata = cipher.decrypt(ciphertext)

    num_padding = gzipdata[-1]
    if num_padding > 0:
        gzipdata = gzipdata[:-num_padding]
    if gzipdata[:2] != b"\x1f\x8b":
        gzipdata = b"\x1f\x8b" + gzipdata

    if mode == "db":
        # 0x10 bytes for random data and 2 bytes for the UTF-16 BOM
        data = gzip.decompress(gzipdata)[0x12:]
    elif mode == "chart":
        data = gzip.decompress(gzipdata)[0x10:]
    else:
        raise ValueError(f"Unknown mode: {mode}")

    if ignore_errors:
        return data.decode(encoding, errors="ignore")

    return data.decode(encoding)


def finale_encrypt(
    mode: str,
    key: Union[str, bytes],
    plaintext: str,
    encoding: str,
) -> bytes:
    if not isinstance(key, bytes):
        key = unhexlify(key.replace(" ", ""))
    if len(key) != 0x10:
        raise ValueError("Invalid key length")

    JUNK = unhexlify("4b67ca1eebc78fb9964f781019bc4903")
    if mode == "db":
        # FEFF is UTF-16 BOM
        encoded = JUNK + b"\xfe\xff" + plaintext.encode(encoding=encoding)
    else:
        encoded = JUNK + plaintext.encode(encoding=encoding)

    gzipdata = gzip.compress(encoded)
    if len(gzipdata) % 0x10 != 0:
        amount = 0x10 - (len(gzipdata) % 0x10)
        padding = amount.to_bytes(1, "big") * amount
        gzipdata += padding

    iv = os.urandom(0x10)
    cipher = AES.new(key, AES.MODE_CBC, iv)

    return iv + cipher.encrypt(gzipdata)



from .maima2tomaisdt import ma2_to_sdt
from .maima2tosimai import ma2_to_simai
from .maisdttomaima2 import sdt_to_ma2
from .maisdttosimai import sdt_to_simai
from .simaitomaima2 import simai_to_ma2
from .simaitomaisdt import simai_to_sdt



import os
import argparse
import re
import sys

from maiconverter.maisdt import MaiSDT
from maiconverter.converter import sdt_to_ma2


def main():
    parser = argparse.ArgumentParser(
        description="Converts sdt to ma2", allow_abbrev=False
    )
    parser.add_argument(
        "path", metavar="Path", type=file_path, help="Path to score file or directory"
    )
    parser.add_argument(
        "--bpm",
        metavar="Song BPM",
        type=float,
        help="BPM of the chart or group of chart's song",
    )
    parser.add_argument(
        "-o",
        "--output",
        metavar="Output directory",
        type=dir_path,
        help="Path to output. Defaults to " + "/path/to/input/output",
    )
    parser.add_argument(
        "-d",
        "--delay",
        metavar="Delay",
        type=float,
        help="Offset a chart by set measures (can be negative)",
    )

    args = parser.parse_args()

    if args.bpm is None:
        print("Error: BPM is required")
        parser.print_help()
        sys.exit(1)

    if args.output is None and os.path.isdir(args.path):
        output_dir = os.path.join(args.path, "output")
    elif args.output is None and not os.path.isdir(args.path):
        output_dir = os.path.join(os.path.dirname(args.path), "output")
    else:
        output_dir = args.output

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    elif os.path.exists(output_dir) and not os.path.isdir(output_dir):
        raise NotADirectoryError(output_dir)

    if os.path.isdir(args.path):
        files = os.listdir(args.path)
        files = [
            file for file in files if os.path.isfile(os.path.join(args.path, file))
        ]

        files = [file for file in files if not re.search(r"\.s..", file) is None]

        for file in files:
            convert_sdt_file(
                os.path.join(args.path, file), output_dir, args.bpm, args.delay
            )
    elif not re.search(r"\.s..", args.path) is None:
        convert_sdt_file(args.path, output_dir, args.bpm, args.delay)
    else:
        print("Error: Not an sdt file")
        sys.exit(1)

    sys.exit(0)


def convert_sdt_file(input_path, output_dir, initial_bpm, delay):
    sdt = MaiSDT()

    file_name = os.path.splitext(os.path.basename(input_path))[0]
    file_ext = ".ma2"

    if os.path.exists(os.path.join(output_dir, file_name + file_ext)):
        print("File {} exists! Skipping".format(file_name + file_ext))
        return

    with open(input_path, "r") as in_f:
        for line in in_f:
            if line in ["\n", "\r\n"]:
                continue

            if re.search(r"\.sr.", input_path) is None:
                sdt.parse_line(line)
            else:
                sdt.parse_srt_line(line)

    ma2 = sdt_to_ma2(sdt, initial_bpm)
    ma2.set_meter(0, 4, 4)
    if delay is not None:
        ma2.offset(delay)

    with open(os.path.join(output_dir, file_name + file_ext), "x") as out_f:
        out_f.write(ma2.export())


# Accepts both file and directory paths
def file_path(string):
    if os.path.exists(string):
        return string.rstrip("/\\")

    raise FileNotFoundError(string)


# Only accepts directory paths
def dir_path(string):
    if os.path.isdir(string):
        return string.rstrip("/\\")

    raise NotADirectoryError(string)


if __name__ == "__main__":
    main()



import os
import argparse
import re
import sys

from maiconverter.simai import SimaiChart
from maiconverter.converter import simai_to_ma2


def main():
    parser = argparse.ArgumentParser(
        description="Converts simai to ma2", allow_abbrev=False
    )
    parser.add_argument(
        "path", metavar="Path", type=file_path, help="Path to score file or directory"
    )
    parser.add_argument(
        "-o",
        "--output",
        metavar="Output directory",
        type=dir_path,
        help="Path to output. Defaults to " + "/path/to/input/output",
    )
    parser.add_argument(
        "-d",
        "--delay",
        metavar="Delay",
        type=float,
        help="Offset a chart by set measures (can be negative)",
    )

    args = parser.parse_args()

    if args.output is None and os.path.isdir(args.path):
        output_dir = os.path.join(args.path, "output")
    elif args.output is None and not os.path.isdir(args.path):
        output_dir = os.path.join(os.path.dirname(args.path), "output")
    else:
        output_dir = args.output

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    elif os.path.exists(output_dir) and not os.path.isdir(output_dir):
        raise NotADirectoryError(output_dir)

    if os.path.isdir(args.path):
        files = os.listdir(args.path)
        files = [
            file for file in files if os.path.isfile(os.path.join(args.path, file))
        ]

        files = [file for file in files if not re.search(r".txt", file) is None]

        for file in files:
            convert_simai_file(os.path.join(args.path, file), output_dir, args.delay)
    elif not re.search(r".txt", args.path) is None:
        convert_simai_file(args.path, output_dir, args.delay)
    else:
        print("Error: Not a simai file")
        sys.exit(1)

    sys.exit(0)


def convert_simai_file(input_path, output_dir, delay):
    file_name = os.path.splitext(os.path.basename(input_path))[0]
    file_ext = ".ma2"

    if os.path.exists(os.path.join(output_dir, file_name + file_ext)):
        print("File {} exists! Skipping".format(file_name + file_ext))
        return

    with open(input_path, "r") as in_f:
        simai_string = in_f.read()

    simai_chart = SimaiChart.from_str(simai_string)

    ma2 = simai_to_ma2(simai_chart)
    if delay is not None:
        ma2.offset(delay)

    with open(os.path.join(output_dir, file_name + file_ext), "x") as out_f:
        out_f.write(ma2.export())


# Accepts both file and directory paths
def file_path(string):
    if os.path.exists(string):
        return string.rstrip("/\\")

    raise FileNotFoundError(string)


# Only accepts directory paths
def dir_path(string):
    if os.path.isdir(string):
        return string.rstrip("/\\")

    raise NotADirectoryError(string)


if __name__ == "__main__":
    main()



def second_to_measure(seconds: float, bpm: float, beats_per_bar: int = 4) -> float:
    beats = bpm * seconds / 60
    return beats / beats_per_bar


def measure_to_second(measure: float, bpm: float, beats_per_bar: int = 4) -> float:
    beats = measure * beats_per_bar
    return 60 * beats / bpm



import argparse
import os
import re

from maiconverter.maicrypt import MaiFinaleCrypt


def main():
    parser = argparse.ArgumentParser(
        description="Encrypts or decrypts a score file or "
        + "a directory containing score files.",
        allow_abbrev=False,
    )

    # python encrypt_decrypt.py encrypt 'KEY HERE IN HEX' /path/to/input
    parser.add_argument(
        "command",
        metavar="Command",
        type=str,
        choices=["encrypt", "decrypt"],
        help="Specify whether to encrypt or decrypt",
    )
    parser.add_argument(
        "key", metavar="Key", type=str, help="16 byte AES key used (whitespace allowed)"
    )
    parser.add_argument(
        "path", metavar="Path", type=file_path, help="Path to score file or directory"
    )
    parser.add_argument(
        "-o",
        "--output",
        metavar="Output directory",
        type=dir_path,
        help="Path to output. Defaults to /path/to/input/output",
    )
    parser.add_argument(
        "--database",
        "-db",
        metavar="Process database/s toggle",
        action="store_const",
        default=False,
        const=True,
    )

    args = parser.parse_args()

    if args.output is None and os.path.isdir(args.path):
        output_dir = os.path.join(args.path, "output")
    elif args.output is None and not os.path.isdir(args.path):
        output_dir = os.path.join(os.path.dirname(args.path), "output")
    else:
        output_dir = args.output

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    elif os.path.exists(output_dir) and not os.path.isdir(output_dir):
        raise NotADirectoryError(output_dir)

    if os.path.isdir(args.path):
        files = os.listdir(args.path)

        files = [
            file for file in files if os.path.isfile(os.path.join(args.path, file))
        ]

        if args.command == "encrypt":
            if args.database:
                files = [file for file in files if not re.search(r".tbl", file) is None]
            else:
                # Only accept ".sdt" ".sct" ".szt" ".srt" files
                files = [file for file in files if not re.search(r".s.t", file) is None]
        else:  # decrypt
            if args.database:
                files = [file for file in files if not re.search(r".bin", file) is None]
            else:
                # Only accept ".sdb" ".scb" ".szb" ".srb" files
                files = [file for file in files if not re.search(r".s.b", file) is None]

        for file in files:
            if args.database:
                handle_db(
                    os.path.join(args.path, file), output_dir, args.command, args.key
                )
            else:
                handle_file(
                    os.path.join(args.path, file), output_dir, args.command, args.key
                )
    else:
        if args.database:
            handle_db(args.path, output_dir, args.command, args.key)
        else:
            handle_file(args.path, output_dir, args.command, args.key)


# input_path is a path to afile not a directory
def handle_file(input_path, output_dir, command, key):
    crypt = MaiFinaleCrypt(key)

    if command == "encrypt":
        file_name = os.path.splitext(os.path.basename(input_path))[0]
        file_ext = os.path.splitext(input_path)[1].replace("t", "b")
        if os.path.exists(os.path.join(output_dir, file_name + file_ext)):
            print("File {} exists! Skipping".format(file_name + file_ext))
            return

        cipher_text = crypt.convert_to_bin(input_path)
        f = open(os.path.join(output_dir, file_name + file_ext), "xb")
        f.write(cipher_text)
        f.close()
    else:
        file_name = os.path.splitext(os.path.basename(input_path))[0]
        file_ext = os.path.splitext(input_path)[1].replace("b", "t")
        if os.path.exists(os.path.join(output_dir, file_name + file_ext)):
            print("File {} exists! Skipping".format(file_name + file_ext))
            return

        plain_text = crypt.convert_to_text(input_path)
        f = open(os.path.join(output_dir, file_name + file_ext), "xb")
        f.write(plain_text)
        f.close()


def handle_db(input_path, output_dir, command, key):
    crypt = MaiFinaleCrypt(key)

    if command == "encrypt":
        file_name = os.path.splitext(os.path.basename(input_path))[0]
        file_ext = ".bin"
        if os.path.exists(os.path.join(output_dir, file_name + file_ext)):
            print("File {} exists! Skipping".format(file_name + file_ext))
            return

        cipher_text = crypt.convert_to_bin(input_path)
        f = open(os.path.join(output_dir, file_name + file_ext), "xb")
        f.write(cipher_text)
        f.close()
    else:
        file_name = os.path.splitext(os.path.basename(input_path))[0]
        file_ext = ".tbl"
        if os.path.exists(os.path.join(output_dir, file_name + file_ext)):
            print("File {} exists! Skipping".format(file_name + file_ext))
            return

        plain_text = crypt.convert_to_text(input_path)
        f = open(os.path.join(output_dir, file_name + file_ext), "xb")
        f.write(plain_text)
        f.close()


# Accepts both file and directory paths
def file_path(string):
    if os.path.exists(string):
        return string.rstrip("/\\")

    raise FileNotFoundError(string)


# Only accepts directory paths
def dir_path(string):
    if os.path.isdir(string):
        return string.rstrip("/\\")

    raise NotADirectoryError(string)


if __name__ == "__main__":
    main()
